import logging

from pywintypes import error as Win32Error
from win32api import OpenProcess
from win32file import CloseHandle
from win32event import SYNCHRONIZE

from .base import ProgramBase


log = logging.getLogger(__name__)

class NTProgram(ProgramBase):


    @property
    def is_running(self):

        pid = self.state.get('pid')
        if not pid:
            return False

        try:
            handle = OpenProcess(SYNCHRONIZE, 0, pid)
            return True
        except Win32Error:
            return False
        finally:
            CloseHandle(handle)




'''
'''
from __future__ import unicode_literals, print_function

from argparse import RawDescriptionHelpFormatter
import logging

from chalmers.program import Program


log = logging.getLogger('chalmers.remove')

def main(args):

    prog = Program(args.name)
    prog.delete()
    log.info("Program %s removed" % args.name)

def add_parser(subparsers):

    parser = subparsers.add_parser('remove',
                                      help='Remove a program definition from chalmers',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter)

    parser.add_argument('name')
    parser.set_defaults(main=main)



'''
Start a program manager that will track and manager chalmers programs

'''

from __future__ import unicode_literals, print_function

from argparse import RawDescriptionHelpFormatter
import logging
import time
from chalmers.event_dispatcher import send_action
from chalmers.program_manager import ProgramManager
from chalmers import errors


log = logging.getLogger('chalmers.manager')

def main(args):

    mgr = ProgramManager(use_color=args.color)

    if args.is_running:
        try:
            result = mgr.send("ping")
        except errors.ChalmersError:
            log.info("Manager is NOT running")
        else:
            log.info("Manager is running with pid %s" % result)
        return
    if args.shutdown:
        result = mgr.send("exitloop")
        log.info("Manager is shutting down")
        return
    log.info("Managing processes")


    try: 
        result = mgr.send("ping")
    except errors.ChalmersError:
        pass
    else:
        raise errors.ChalmersError("An instance of chalmers manager is already running (pid: %s)" % result)

    mgr.start_all()
    mgr.start_listener()

    #Workaround so keyboard interupt works on win32
    try:
        while mgr.is_listening:
            time.sleep(1)
    except BaseException as err:
        mgr.send('exitloop')
        raise

    print("Finished manager")



def add_parser(subparsers):
    parser = subparsers.add_parser('manager',
                                   help='Manage Chalmers programs',
                                   description=__doc__,
                                   formatter_class=RawDescriptionHelpFormatter)

    parser.add_argument("--shutdown", action='store_true',
                help="Exit manager")
    parser.add_argument("--is-running", action='store_true',
                help="Check if the manager is running")
    parser.set_defaults(main=main)



from chalmers import errors
import shlex

def make_definition(data):
    """
    Turn the 'data' dict into a standard definition 
    """
    if 'name' not in data:
        raise errors.ChalmersError("Program definition is missing the field 'name'" % data)

    if 'command' not in data:
        raise errors.ChalmersError("Program definition %(name)s is missing the field 'command'" % data)

    if 'extends' in data:
        if isinstance(data['extends'], str):
            data['extends'] = [data['extends']]

    if isinstance(data['command'], str):
        data['command'] = shlex.split(data['command'])


    if not isinstance(data['command'], list):
        raise errors.ChalmersError("The field 'command' must be a list or a string got %r" % type(data['command']))

    return data





'''
'''
from __future__ import unicode_literals, print_function

from argparse import FileType
import logging
from os import path
import os

import yaml

from chalmers.config import dirs
from chalmers.utils.definition import make_definition


log = logging.getLogger('chalmers.import')



def main(args):

    import_data = yaml.safe_load(args.input)
    groups = {k['group']['name']:k['group'] for k in import_data if 'group' in k}
    programs = {k['program']['name']:k['program'] for k in import_data if 'program' in k}

    for group in groups.values():
        group_dir = path.join(dirs.user_data_dir, 'groups')
        if not path.isdir(group_dir): os.makedirs(group_dir)

        group_path = path.join(group_dir, '%s.yaml' % group['name'])
        log.info("Writing group %s to %s" % (group['name'], group_path))
        with open(group_path, 'w') as gf:
            yaml.safe_dump(group, gf, default_flow_style=False)

    for program in programs.values():
        program = make_definition(program)
        program_dir = path.join(dirs.user_data_dir, 'programs')
        if not path.isdir(program_dir): os.makedirs(program_dir)

        program_path = path.join(program_dir, '%s.yaml' % program['name'])
        log.info("Writing program %s to %s" % (program['name'], program_path))

        with open(program_path, 'w') as pf:
            yaml.safe_dump(program, pf, default_flow_style=False)

def add_parser(subparsers):
    parser = subparsers.add_parser('import',
                                      help='[IN DEVELOPMENT] Batch import many commands from a yaml file',
                                      description=__doc__)

    parser.add_argument('input', type=FileType('r'))
    parser.set_defaults(main=main)



import abc
import logging
import os
import sys
import traceback

import servicemanager
import win32event, win32serviceutil, win32service, win32api
from win32file import ReadFile, WriteFile
from win32pipe import CreateNamedPipe, ConnectNamedPipe, DisconnectNamedPipe
from win32pipe import PIPE_ACCESS_DUPLEX, PIPE_TYPE_MESSAGE, PIPE_WAIT, PIPE_UNLIMITED_INSTANCES
from win32serviceutil import ServiceFramework


log = logging.getLogger(__name__)

class WindowsService(object, ServiceFramework):
    __metaclass__ = abc.ABCMeta

    def __init__(self, args):
        try:

            self._svc_name_ = args[0]
            self._svc_display_name_ = args[0]

            ServiceFramework.__init__(self, args)
            self.stop_event = win32event.CreateEvent(None, 0, 0, None)

        except Exception:
            self.log("Error in WindowsService.__init__")
            self.log(traceback.format_exc())
            raise

    def log(self, msg):
        'Log to the NTEventlog'
        servicemanager.LogInfoMsg(str(msg))

    def sleep(self, sec):
        win32api.Sleep(sec * 1000, True)


    def SvcDoRun(self):
        self.log('start')

        self.ReportServiceStatus(win32service.SERVICE_START_PENDING)
        try:
            self.ReportServiceStatus(win32service.SERVICE_RUNNING)
            self.log('start')
            self.start()
            self.log('wait')
            win32event.WaitForSingleObject(self.stop_event, win32event.INFINITE)
            self.log('done')
        except Exception:
            self.log("Error in WindowsService.SvcDoRun")
            self.log(traceback.format_exc())
            self.SvcStop()


    def SvcStop(self):
        self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
        self.log('stopping')
        self.stop()
        self.log('stopped')
        win32event.SetEvent(self.stop_event)
        self.ReportServiceStatus(win32service.SERVICE_STOPPED)





import unittest
from chalmers.program import Program


class Test(unittest.TestCase):

    def test_Init(self):
        p = Program('name', load=False)
        self.assertEqual(p.name, 'name')

    def test_(self):
        pass


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



"""
Install a crontab rule to run at system boot 
"""
from __future__ import unicode_literals, print_function

import logging
from subprocess import Popen, check_output, CalledProcessError, PIPE
import sys

from chalmers import errors


python_exe = sys.executable
chalmers_script = sys.argv[0]
chalmers_tab_entry = '@reboot %s %s start -a' % (python_exe, chalmers_script)

log = logging.getLogger('chalmers.reboot')

def get_crontab():
    try:
        output = check_output(['crontab', '-l']).strip()
    except CalledProcessError:
        raise errors.ChalmersError("Could not read crontab")

    return output.split('\n')

def set_crontab(tab):

    new_cron_tab = '\n'.join(tab) + '\n'

    p0 = Popen(['crontab'], stdin=PIPE)
    p0.communicate(input=new_cron_tab)

def main(args):
    tab_lines = get_crontab()

    if chalmers_tab_entry in tab_lines:
        log.info("Chalmers crontab instruction already exists")
    else:
        log.info("Adding chalmers instruction to crontab")
        tab_lines.append(chalmers_tab_entry)

        set_crontab(tab_lines)

        log.info("All chalmers programs will now run on boot")

def main_uninstall(args):

    tab_lines = get_crontab()

    if chalmers_tab_entry in tab_lines:
        log.info("Removing chalmers instruction from crontab")
        tab_lines.remove(chalmers_tab_entry)

        set_crontab(tab_lines)

    else:
        log.info("Chalmers crontab instruction does not exist")

def main_status(args):

    tab_lines = get_crontab()

    if chalmers_tab_entry in tab_lines:
        log.info("Chalmers is setup to start on boot")
    else:
        log.info("Chalmers will not start on boot")




"""
Chalmers program object
"""
from __future__ import absolute_import, unicode_literals, print_function

import abc
from glob import glob
import logging
from threading import Lock, Event
from os import path
import os
import signal
from subprocess import Popen, STDOUT
import time

import yaml

from chalmers import errors
from chalmers.config import dirs
from chalmers.event_dispatcher import EventDispatcher, send_action
import sys
import psutil


log = logging.getLogger(__name__)

def str_replace(data):
    """
    String substitution of `data` dict
    """
    for key, value in data.items():
        if isinstance(value, (str, unicode)):
            data[key] = value.format(**data)

class ProgramBase(EventDispatcher):
    """
    Object that represents a long running process
    
    This program option may represent a program that is 
    running in another process
    or a program that is running in the current process.
    """
    __metaclass__ = abc.ABCMeta


    OPTIONS = [('Primary Options',
                ['name', 'command', 'templates']),
               ('Output',
                ['stdout', 'stderr', 'daemon_log', 'redirect_stderr']),
               ('Process Controll',
                ['retries', 'exitcodes', 'stopwaitsecs',
                 'stopsignal', 'startsecs' ])
              ]

    DEFAULTS = {
                'retries': 3,
                'exitcodes': [0],
                'startsecs': 10,
                'stopwaitsecs': 10,
                'stopsignal': signal.SIGTERM,
                'log_dir': dirs.user_log_dir,

                'redirect_stderr': False,
                'stdout': '{log_dir}/{name}.stdout.log',
                'stderr': '{log_dir}/{name}.stderr.log',
                'daemon_log': '{log_dir}/{name}.daemon.log',
                }

    @abc.abstractproperty
    def is_running(self): pass

    def start_as_service(self):
        """
        Run this program in a new background process
        
        chalmers manager must be running
        """
        from ..program_manager import ProgramManager
        send_action(ProgramManager.NAME, 'start', self.name)

    @property
    def definition_filename(self):
        'The file name where the run defn is stored in'
        return path.join(dirs.user_data_dir, 'programs', '%s.yaml' % self.name)

    @property
    def state_filename(self):
        'The file name where current program state is stored'
        return path.join(dirs.user_data_dir, 'state', '%s.yaml' % self.name)

    @property
    def lock_filename(self):
        'The file name where current program state is stored'
        return path.join(dirs.user_data_dir, 'lock', '%s' % self.name)


    def __init__(self, name, load=True):
        self._name = name

        lock_dir = path.dirname(self.lock_filename)

        if not path.isdir(lock_dir):
            os.makedirs(lock_dir)

        self.file_lock = Lock()

        EventDispatcher.__init__(self)
        self.finished_event = Event()

        self.raw_data = {}
        self.data = {}
        self.state = {}
        self._p0 = None
        if load:
            self.reload()
            self.reload_state()


    @property
    def name(self):
        return self._name

    @classmethod
    def create(cls, name, defn, state=None):
        """
        Create a new program object
        """
        prog = cls(name, False)
        prog.raw_data = defn
        prog.state = state or {}
        prog.mk_data()

        return prog

    @property
    def stopsignal(self):
        stopsignal = self.data['stopsignal']

        if isinstance(stopsignal, basestring) and hasattr(signal, stopsignal):
            stopsignal = getattr(signal, stopsignal)
        if not isinstance(stopsignal, int):
            log.warning("Stopsignal %s is not a valid signal for this platform" % stopsignal)
            log.warning("Signal SIGTERM (%s) will be used" % signal.SIGTERM)
        return stopsignal

    def save(self):
        """
        Save the program definition to file
        """
        defn_dir = path.dirname(self.definition_filename)

        if not path.isdir(defn_dir):
            os.makedirs(defn_dir)

        # Force check of stopsignal
        self.stopsignal

        with open(self.definition_filename, 'w') as df:
            yaml.safe_dump(self.raw_data, df, default_flow_style=False)

    def save_state(self):
        """
        Save the program state to file
        """
        state_dir = path.dirname(self.state_filename)

        if not path.isdir(state_dir):
            os.makedirs(state_dir)

        with open(self.state_filename, 'w') as df:
            log.debug("Saving state of program %s to %s" % (self.name, self.state_filename))
            yaml.safe_dump(self.state, df, default_flow_style=False)

    def reload_state(self):
        """
        Replace the state in memory with the state in the state file
        """

        log.debug("Reload state from file %s" % self.state_filename)
        if path.isfile(self.state_filename):
            with open(self.state_filename) as sf:
                self.state = yaml.safe_load(sf)

                if self.state is None:
                    log.debug("Statefile returned none")
        else:
            log.debug("Statefile does not exist")
            self.state = {}

    def reload(self):
        """
        Replace the program definition in memory with the definition 
        from the defn file
        """

        if not path.isfile(self.definition_filename):
            msg = "Program %s does not exist (no definition file %s)"
            raise errors.ProgramNotFound(msg % (self.name, self.definition_filename))

        with open(self.definition_filename) as df:
            self.raw_data = yaml.safe_load(df)

        self.mk_data()

    @classmethod
    def load_template(cls, template_name):
        """
        Load a template from file
        """

        template_path = path.join(dirs.user_data_dir, 'template', '%s.yaml' % template_name)

        if not path.isfile(template_path):
            return {}

        with open(template_path, 'r') as gf:
            return yaml.safe_load(gf)

    def mk_data(self):
        """
        Transform the 'raw_data' from the definition into
        the used data
        """
        self.data = self.DEFAULTS.copy()

        for template in self.raw_data.get('extends', []):
            template_data = self.load_template(template)
            self.data.update(template_data)

        self.data.update(self.raw_data)

        str_replace(self.data)

        if self.data.get('redirect_stderr'):
            self.data.pop('stderr')

    @property
    def is_paused(self):
        return self.state.get('paused')


    def update_definition(self, *E, **F):
        'Update the program definition'
        with self.file_lock:
            self.reload()
            self.raw_data.update(*E, **F)
            self.save()

    def update_state(self, *E, **F):
        'Update the program state'
        with self.file_lock:

            self.reload_state()
            self.state.update(*E, **F)
            log.debug("Update state with info: %r" % dict(*E, **F))
            self.save_state()

    def start(self, daemon=True):
        """
        Start this process 
        :param daemon: if true, start the process as a background process
        
        this will fail if the process is already running
        """
        if self.is_running:
            raise errors.StateError("Process is already running")

        if not daemon:
            self.start_sync()
        else:
            self.start_as_service()

    def start_sync(self):
        """
        Syncronously run this program in this process
        """

        if 'daemon_log' in self.data:
            self.log_to_daemonlog()

        self.start_listener()

        self.update_state(pid=os.getpid())

        try:
            self.keep_alive()
        except errors.StopProcess:
            self._terminate()
        finally:
            self.update_state(pid=None)
            self.finished_event.set()
            self._running = False
            if self._listener:
                try:
                    send_action(self.name, 'exitloop')
                except:
                    pass


    def dispatch_terminate(self, timeout=None):
        'Action for event listener'
        self._terminate()
        self._running = False
        if not self.finished_event.wait(timeout):
            raise errors.ChalmersError("Timed out waiting for program %s to finish" % self.name)

    def _terminate(self):
        """
        Terminate this process, 
        This function may only be called by the process that called 'start_sync'
        
         
        """

        stopsignal = self.stopsignal

        log.info('Stop Process Requested')
        self._terminating = True
        if self._p0:
            log.info('Sending signal %s to process %s' % (stopsignal, self._p0.pid))
            kill_tree(self._p0.pid, stopsignal)
        elif self._p0 is None:
            raise errors.ChalmersError("This process did not start this program, can not call _terminate")

    def keep_alive(self):
        """
        """
        self._p0 = False

        if self.data['redirect_stderr']:
            stderr = STDOUT
        else:
            stderr = open(self.data['stderr'], 'a')

        stdout = open(self.data['stdout'], 'a')

        self._terminating = False
        for i in range(self.data['retries'] + 1):
            start = time.time()
            if i:
                log.info('Retry command (%i of %i)' % (i, self.data['retries']))
            env = os.environ.copy()
            env.update({k:str(v) for (k, v) in self.data.get('env', {}).items()})
            log.info("Running Command: %s" % ' '.join(self.data['command']))
            try:
                self._p0 = Popen(self.data['command'], stdout=stdout, stderr=stderr, env=env)
            except OSError as err:
                log.error('Program %s could not be started with popen' % self.name)
                self.update_state(child_pid=None, exit_status=1,
                                  reason='OSError running command "%s"' % self.data['command'][0])
                return
            except:
                log.exception('Exception in keep_alive')
                self.update_state(child_pid=None, exit_status=1,
                                  reason='There was an unknown exception opening command (check logs)')
                return



            log.info('Program started with pid %s' % self._p0.pid)
            self.update_state(child_pid=self._p0.pid, reason=None, exit_status=None,
                              start_time=time.time())

            try:
                status = self._p0.wait()
            except KeyboardInterrupt:
                log.error('Program %s was interrupted by user' % self.name)
                kill_tree(self._p0.pid, signal.SIGTERM)
                self.update_state(child_pid=None, exit_status=None, reason='Interrupted by user')
                raise

            self._p0 = False

            uptime = time.time() - start

            log.info('Command Exited with status %s' % status)
            log.info(' + Uptime %s' % uptime)

            if self._terminating:
                reason = "Terminated at user request"
                status = None
            elif uptime < self.data['startsecs']:
                reason = 'Program did not successfully start'
            elif status in self.data['exitcodes']:
                reason = "Program exited gracefully"
            else:
                reason = "Program exited unexpectedly with code %s" % (status)

            self.update_state(child_pid=None, exit_status=status,
                              reason=reason)

            if self._terminating:
                break

            if status in self.data['exitcodes']:
                break

        log.debug("Exiting keep alive function")

    def delete(self):
        """
        Remove this program definition
        """
        if self.is_running:
            raise errors.ChalmersError("Can not remove running program (must be stopped)")

        if path.isfile(self.definition_filename):
            os.unlink(self.definition_filename)

        if path.isfile(self.state_filename):
            os.unlink(self.state_filename)


    @classmethod
    def find_for_user(cls):
        'Find all programs this user has defined'
        program_glob = path.join(dirs.user_data_dir, 'programs', '*.yaml')
        for filename in glob(program_glob):
            basename = path.basename(filename)
            name = path.splitext(basename)[0]
            yield cls(name)


    @classmethod
    def start_all(cls, start_paused=False):
        'Start all user defined programs'
        log.info("Starting all programs")

        for prog in cls.find_for_user():
            if not start_paused and prog.is_paused:
                log.info(" - Program %s is paused" % prog.name)
            elif not prog.is_running:
                log.info(" + Starting program %s" % prog.name)
                prog.start(daemon=True)
            else:
                log.info(" - Programs %s is already running" % prog.name)

    @property
    def is_ok(self):
        if self.is_running:
            return True
        elif self.state.get('exit_status') is None:
            return True
        elif self.state.get('exit_status') in self.data['exitcodes']:
            return True
        return False
    @property
    def text_status(self):
        'A text status of the current program'

        if self.is_running:
            return 'RUNNING'
        elif self.is_paused:
            return 'PAUSED'
        elif self.is_ok:
            return 'STOPPED'
        else:
            return 'ERROR'

    def log_to_daemonlog(self):
        self._log_stream = open(self.data['daemon_log'], 'a')
        hdlr = logging.StreamHandler(self._log_stream)
        hdlr.setLevel(logging.INFO)
        fmt = logging.Formatter("[%(asctime)s] %(levelname)s:%(message)s")
        hdlr.setFormatter(fmt)
        logging.getLogger('chalmers').addHandler(hdlr)



    def stop(self):
        """
        Stop this program
        """

        if not self.is_running:
            raise errors.StateError("Program is not running")

        send_action(self.name, 'terminate', timeout=self.data['stopwaitsecs'])


    def restart(self):
        if self.is_running:
            print("Stopping program %s ..." % self.name, end=''); sys.stdout.flush()
            try:
                self.stop()
            except errors.StateError as err:
                log.error(err.message)
            else:
                print("stopped ")
        else:
            print("Program %s is already stopped" % self.name)

        print("Starting program %s ... " % self.name, end=''); sys.stdout.flush()
        self.reload_state()
        self.start()
        print("restarted")

def kill_tree(pid, sig):
    'Kill all processes and child processes'
    try:
        parent = psutil.Process(pid)
    except psutil.NoSuchProcess:
        log.warn("Kill failed, process with pid %s does not appear to be running" % pid)
        return 
    children = parent.get_children(recursive=True)
    os.kill(pid, sig)

    for child in children:
        if child.is_running():
            child.kill()



import logging

from chalmers.event_dispatcher import send_action
from chalmers.program import Program
from chalmers.program_manager import ProgramManager
from chalmers.utils.logutil import setup_logging
from chalmers.windows.service_base import WindowsService


class ChalmersService(WindowsService):
    """
    Run the chalmers manager process as a windows service
    """
    def start(self):

        setup_logging(logging.INFO, False, 'service.log')
        self.mgr = mgr = ProgramManager(use_color=False)
        mgr.start_all()
        mgr.listen()

    def stop(self):

        self.log("Stop Called")
        for prog in Program.find_for_user():
            if prog.is_running:
                self.log("Stopping program %s" % prog.name)
                prog.stop()

        self.log("Sending chalmers manager exit signal")
        send_action(ProgramManager.NAME, "exitloop")





__version__ = '0.0.1'
import signal

defaults = {
            'retries': 3,
            'exitcodes': [0],
            'startsecs': 10,
            'stopwaitsecs': 10,
            'stopsignal': signal.SIGTERM,
            }

def make_program_data(data):
    new_data = defaults.copy()
    new_data.update(data)
    return new_data



import getpass
import logging
from os.path import abspath
import sys

from chalmers.scripts import service as service_script
import win32api, win32serviceutil, win32service


log = logging.getLogger(__name__)

def InstallService(serviceName, displayName, startType=win32service.SERVICE_DEMAND_START,
                    serviceDeps=None,
                    errorControl=win32service.SERVICE_ERROR_NORMAL,
                    userName=None, password=None,
                    description=None):
    serviceType = win32service.SERVICE_WIN32_OWN_PROCESS

    script = abspath(service_script.__file__)
    commandLine = "%s %s" % (sys.executable, script)

    hscm = win32service.OpenSCManager(None, None, win32service.SC_MANAGER_ALL_ACCESS)

    try:
        hs = win32service.CreateService(hscm,
                                serviceName,
                                displayName,
                                win32service.SERVICE_ALL_ACCESS,  # desired access
                    serviceType,  # service type
                    startType,
                    errorControl,  # error control type
                    commandLine,
                    None,
                    0,
                    serviceDeps,
                    userName,
                    password)

        if description is not None:
            try:
                win32service.ChangeServiceConfig2(hs, win32service.SERVICE_CONFIG_DESCRIPTION, description)
            except NotImplementedError:
                pass  # # ChangeServiceConfig2 and description do not exist on NT

        win32service.CloseServiceHandle(hs)
    finally:
        win32service.CloseServiceHandle(hscm)

def get_service_name(userName):
    "Return the service name for the given user"
    simpleUserName = userName.rsplit('\\', 1)[-1]
    svc_name = 'chalmers:manager:%s' % simpleUserName
    return svc_name

def is_running(userName=getpass.getuser()):
    "Test if the chalmers sevice is running for 'userName'"
    svc_name = get_service_name(userName)
    try:
        status = win32serviceutil.QueryServiceStatus(svc_name)
    except win32api.error:
        return False
    return status[1] == win32service.SERVICE_RUNNING

def is_installed(userName=getpass.getuser()):
    "Test if the chalmers sevice is installed for 'userName'"
    svc_name = get_service_name(userName)
    try:
        win32serviceutil.QueryServiceStatus(svc_name)
        return True
    except win32api.error:
        return False

def instart(userName, password):
    ''' Install and  Start (auto) a the chalmers service
    '''

    simpleUserName = userName.rsplit('\\', 1)[-1]
    svc_name = get_service_name(userName)
    display_name = 'Chalmers service manager for user %s' % simpleUserName

    win32api.SetConsoleCtrlHandler(lambda x: True, True)

    InstallService(
        svc_name,
        display_name,
        startType=win32service.SERVICE_AUTO_START,
        userName=userName,
        password=password,
    )

    log.info('Install OK')
    win32serviceutil.StartService(svc_name)
    log.info('Start OK')





'''
'''
from __future__ import unicode_literals, print_function

from argparse import FileType
import logging
import sys

import yaml

from chalmers.program import Program


log = logging.getLogger('chalmers.export')



def main(args):

    export_data = []

    for prog in Program.find_for_user():
        export_data.append({'program': prog.raw_data})

    yaml.safe_dump(export_data, args.output, default_flow_style=False)

def add_parser(subparsers):
    parser = subparsers.add_parser('export',
                                      help='[IN DEVELOPMENT] Export current configuration to be installed with the "import" command',
                                      description=__doc__)

    parser.add_argument('names', nargs='*')
    parser.add_argument('-o', '--output', type=FileType('w'), default=sys.stdout)
    parser.set_defaults(main=main)



'''
[Un]Install chalmers so that it will run at system boot

This set is required on win32 platforms:

eg:
   
    chalmers install-service
    
'''
from __future__ import unicode_literals, print_function
import getpass
import os
from argparse import RawDescriptionHelpFormatter

if os.name == 'nt':
    from . import _install_service_nt as svs
else:
    from . import _install_service_posix as svs


def add_parser(subparsers):
    parser = subparsers.add_parser('install-service',
                                      help='Install chalmers as a service',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter)
    parser.add_argument('-u', '--username', default=getpass.getuser(),
                        help='User account to run chalmers in (default: %(default)s)')
    parser.set_defaults(main=svs.main)

    #####  #####  #####  #####  #####  #####  #####  #####  #####  #####
    #####  #####  #####  #####  #####  #####  #####  #####  #####  #####
    parser = subparsers.add_parser('uninstall-service',
                                      help='Uninstall chalmers as a service',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter)
    parser.add_argument('-u', '--username', default=getpass.getuser(),
                        help='User account to run chalmers in (default: %(default)s)')
    parser.set_defaults(main=svs.main_uninstall)
    #####  #####  #####  #####  #####  #####  #####  #####  #####  #####
    #####  #####  #####  #####  #####  #####  #####  #####  #####  #####
    parser = subparsers.add_parser('service-status',
                                      help='Check the status of the service',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter)
    parser.add_argument('-u', '--username', default=getpass.getuser(),
                        help='User account to run chalmers in (default: %(default)s)')

    parser.set_defaults(main=svs.main_status)




import sys
import logging
from chalmers.utils.colors import color
import traceback

class FormatterWrapper(object):

    def __init__(self, formatter, prefix='', suffix=''):
        self.prefix = prefix
        self.suffix = suffix
        self.formatter = formatter

    def format(self, record):
        result = self.formatter.format(record)
        if not isinstance(result, list):
            result = [result]

        if self.prefix:
            result.insert(0, self.prefix)
        if self.suffix:
            result.append(self.suffix)
        return result

    @classmethod
    def wrap(cls, handler, prefix='', suffix=''):
        old_formatter = handler.formatter or logging.Formatter()
        new_formatter = cls(old_formatter, prefix, suffix)
        handler.setFormatter(new_formatter)


class ColorFormatter(object):

    COLOR_MAP = {'ERROR': (color.BOLD, color.RED),
                 'WARNING': (color.BOLD, color.YELLO),
                 'DEBUG': (color.BOLD, color.BLUE),
                 }

    def color_map(self, header, level):
        header = '[%s]' % header
        if level in self.COLOR_MAP:
            return color(header, self.COLOR_MAP[level])
        else:
            return header

    def __init__(self, isatty=True, short_tb=()):
        self.isatty = isatty
        self.short_tb = short_tb

    def format(self, record):
        if record.levelno == logging.INFO:
            header = None
            message = record.getMessage()
        else:
            if record.exc_info:
                err = record.exc_info[1]
                header = type(err).__name__
                result = [self.color_map(header, record.levelname)]

                if hasattr(err, 'message'):
                    result.append(str(err.message))
                elif err.args:
                    result.append(str(err.args[0]))

                if not isinstance(err, self.short_tb):
                    message = ''.join(traceback.format_exception(*record.exc_info))
                    result.append('\n' + message)
                return result
#                 else:
#                     message = str(err)
            else:
                header = record.levelname.lower()
                message = record.getMessage()

        if header:
            header = self.color_map(header, record.levelname)
            return [header, '%s' % message]
        else:
            return message

class MyStreamHandler(logging.Handler):
    def __init__(self, color=None, level=logging.INFO, short_tb=()):

        logging.Handler.__init__(self, level=level)

        if color is None:
            color = sys.stdout.isatty()

        self.setFormatter(ColorFormatter(color, short_tb=short_tb))

    def emit(self, record):

        if not self.filter(record):
            return

        fmt = self.format(record)

        if record.levelno == logging.INFO:
            stream = sys.stdout
        else:
            stream = sys.stderr

        if isinstance(fmt, (list, tuple)):
            for item in fmt:
                if isinstance(item, color):
                    with item(stream) as text:
                        stream.write(text)
                else:
                    stream.write(item)
                stream.write(' ')
        else:
            stream.write(fmt)

        stream.write('\n')

def main():
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    h = MyStreamHandler(True, logging.DEBUG)
    logger.addHandler(h)

    logger.debug("DEBUG")
    logger.info("INFO")
    logger.warn("WARN")
    logger.error("ERROR")

    FormatterWrapper.wrap(h, prefix=color('prefix |', [color.WHITE, color.BACKGROUND_COLORS[0]]))

    logger.debug("DEBUG")
    logger.info("INFO")
    logger.warn("WARN")
    logger.error("ERROR")

if __name__ == '__main__':
    main()



'''List the all the defined programs

eg: 

    $ chalmers list
    server1                             RUNNING         pid 19278 , uptime 0:10:01

    
'''
from __future__ import unicode_literals, print_function

from argparse import RawDescriptionHelpFormatter
from datetime import timedelta
import logging
import time

from chalmers.program import Program


log = logging.getLogger(__name__)

def main(args):

    programs = list(Program.find_for_user())

    for prog in programs:

        if prog.is_running:
            start_time = prog.state.get('start_time')
            td = str(timedelta(seconds=(time.time() - start_time))).rsplit('.', 1)[0]
            ctx = (prog.data['name'][:35], prog.text_status,
                   prog.state.get('child_pid'),
                   td,
                )

            print('%-35s %-15s pid %-6s, uptime %s' % ctx)
        else:
            ctx = (prog.data['name'][:35], prog.text_status,
                   prog.state.get('reason')
                  )

            print('%-35s %-15s %s' % ctx)

    if not programs:
        print('No programs added')


def add_parser(subparsers):
    parser = subparsers.add_parser('list',
                                      help='List registered programs',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter)

    parser.set_defaults(main=main)



from contextlib import contextmanager
import itertools
import logging
from multiprocessing import Process, Manager
import random
import sys

from chalmers.event_dispatcher import EventDispatcher
from chalmers.program import Program
from chalmers.utils.handlers import FormatterWrapper, MyStreamHandler
from chalmers.utils.colors import color

log = logging.getLogger(__name__)

class ProgramManager(EventDispatcher):
    """
    Manages chalmers programs with multiprocessing
    
    Listens to 'start' events to start new programs
    """

    NAME = 'chalmers_manager'

    COLOR_CODES = range(40, 48) + [100, 102, 104, 105, 106]
    random.shuffle(COLOR_CODES)

    def __init__(self, exit_on_first_failure=False, use_color=None):
        EventDispatcher.__init__(self)
        self.manager = Manager()
        self.processes = []

        if use_color is None:
            use_color = sys.stdout.isatty()

        self.use_color = use_color
        self.bg_colors = itertools.cycle(color.BACKGROUND_COLORS)
        self.exit_on_first_failure = exit_on_first_failure


    @property
    def name(self):
        return self.NAME

    def dispatch_start(self, name):
        log.info("Managing Program %s" % name)
        p = Process(target=start_program,
                    name='start_program:%s' % name,

                    args=(name,),
                    kwargs={'color_id': self.use_color and next(self.bg_colors)})

        p.start()
        self.processes.append(p)

    def start_all(self):
        for prog in Program.find_for_user():
            if not prog.is_paused:
                self.dispatch_start(prog.name)
            else:
                log.info("Not starting program %s (it is paused)" % (prog.name))

    @contextmanager
    def cleanup(self, prog):
        try:
            yield
        except KeyboardInterrupt:
            log.error('KeyboardInterrupt')
        finally:
            if not prog.is_ok:
                log.info("Program manager letting program fail")


def start_program(name, color_id=None):

    logger = logging.getLogger('chalmers')

    prefix = '[%s]' % name
    if color_id is not None:
        prefix = color(prefix, [color_id, color.WHITE])

    if not logger.handlers:
        shndlr = MyStreamHandler(color_id is not None)
        shndlr.setLevel(logging.INFO)
        logger.setLevel(logging.INFO)
        logger.addHandler(shndlr)

    for h in logger.handlers:
        FormatterWrapper.wrap(h, prefix=prefix)
    prog = Program(name)

    prog.start_sync()




"""
Chalmers error classes
"""

class ChalmersError(Exception):
    pass

class ProgramNotFound(ChalmersError):
    pass

class StateError(ChalmersError):
    pass


class ShowHelp(object):
    pass



class StopProcess(Exception):
    pass



from chalmers.windows.chalmers_service import ChalmersService

def main():
    import servicemanager
    servicemanager.Initialize(None, None)
    servicemanager.PrepareToHostSingle(ChalmersService)
    servicemanager.StartServiceCtrlDispatcher()


if __name__ == "__main__":
	main()
	


'''
Set a definition variable for a program

To update a programs environment 
run:

    chalmers set server1 env.PORT=5000
'''
from __future__ import unicode_literals, print_function

import logging

from chalmers.program import Program
from chalmers.utils import try_eval, set_nested_key
from chalmers import errors
from argparse import RawDescriptionHelpFormatter


log = logging.getLogger('chalmers.set')

def split_try_eval(item):

    if '=' not in item:
        raise TypeError("Chalmers can not set '%s' as a definition variable\n"
                        "The parameter must contain an '='" % (item))
    key, value = item.split('=', 1)
    return key, try_eval(value)

def main(args):

    proc = Program(args.name)

    if proc.is_running:
        log.warning("Program is running: Updates will not be reflected until a restart is done")

    for key, value in args.items:
        if key == 'name':
            raise errors.ChalmersError("Can not set program name")

        set_nested_key(proc.raw_data, key, value)
        log.info("Set '%s' to %r for program %s" % (key, value, args.name))

    proc.mk_data()
    proc.save()

    log.info("done")


def add_parser(subparsers):
    parser = subparsers.add_parser('set',
                                      help='Set a variable in the program definition',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter)

    parser.add_argument('name', metavar='PROG')
    parser.add_argument('items', nargs='+', metavar='KEY=VALUE', type=split_try_eval)
    parser.set_defaults(main=main)



'''
Run a program

eg:
    chalmers run --name server1 -- python /path/to/myserver.py
or:
    chalmers run --name server1 -c "python /path/to/myserver.py"
    
    
'''
from __future__ import unicode_literals, print_function

import logging
from os import path
import os

from chalmers import errors
from chalmers.config import dirs
from chalmers.program import Program
from argparse import RawDescriptionHelpFormatter
import shlex


log = logging.getLogger('chalmers.add')

def main(args):

    if args.cmd and args.command:
        raise errors.ChalmersError('Unknow arguments %r' % args.command)
    if args.cmd:
        args.command = args.cmd

    program_dir = path.join(dirs.user_data_dir, 'programs')

    if not args.name:
        args.name = args.command[0]

    if not path.isdir(program_dir):
        os.makedirs(program_dir)

    program_defn = path.join(program_dir, '{args.name}.yaml'.format(args=args))
    if path.isfile(program_defn):
        raise errors.ChalmersError("Program with name '{args.name}' already exists.  \n"
                                   "Use the -n/--name option to change the name or \n"
                                   "Run 'chalmers remove {args.name}' to remove it \n"
                                   "or 'chalmers set' to update the parameters".format(args=args))

    definition = {
                    'name': args.name,
                    'command': args.command,
    }

    if args.stdout:
        definition['stdout'] = args.stdout

    if args.daemon_log:
        definition['daemon_log'] = args.daemon_log

    if args.redirect_stderr is not None:
        definition['redirect_stderr'] = args.redirect_stderr

    if args.stderr is not None:
        definition['stderr'] = args.stderr

    state = {'paused': args.paused}

    prog = Program.create(args.name, definition, state)
    prog.save_state()

    if not args.paused:
        log.info('Starting program {args.name}'.format(args=args))
        prog.start(daemon=args.daemon)

    if args.daemon:
        prog.save()
        log.info('Added program {args.name}'.format(args=args))
    else:
        log.info('Program {args.name} exited'.format(args=args))


def add_parser(subparsers):
    parser = subparsers.add_parser('run',
                                      help='Manage a command to run',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter
                                      )
    #===============================================================================
    #
    #===============================================================================
    group = parser.add_argument_group('Starting State') \
                  .add_mutually_exclusive_group()

    group.add_argument('--start', action='store_false', dest='paused', default=False,
                       help="Start program automatically (default)")
    group.add_argument('--paused', action='store_true', dest='paused',
                       help="don't start program automatically")

    group.add_argument('--daemon', action='store_true', default=True,
                       help="Run command in the background")
    group.add_argument('-w', '--wait', '--no-daemon', action='store_false', dest='daemon',
                       help="Run command in the foreground")
    #===========================================================================
    #
    #===========================================================================
    group = parser.add_argument_group('Process Output:')
    group.add_argument('--stdout')
    group.add_argument('--stderr')
    group.add_argument('--daemon-log')
    group.add_argument('--redirect-stderr', action='store_true')
    #===========================================================================
    #
    #===========================================================================
    parser.add_argument('-n', '--name',
                        help='Set the name of this program for future chalmers commands')
    parser.add_argument('command', nargs='*', metavar='COMMAND',
                        help='Command to run')
    split = lambda item: shlex.split(item, posix=os.name == 'posix')

    parser.add_argument('-c', metavar='COMMAND', type=split, dest='cmd',
                        help='Command to run')
    parser.set_defaults(main=main, state='pause')



from __future__ import unicode_literals, print_function

from chalmers.utils import appdirs


dirs = appdirs.AppDirs('chalmers', 'srossross')

def set_relative_dirs(root):
    'Set the application directory relative root'
    global dirs
    dirs = appdirs.RelativeAppDirs(root)



import logging
from logging.handlers import RotatingFileHandler
from os import makedirs
from os.path import join, exists

from chalmers.config import dirs
from chalmers.utils.handlers import MyStreamHandler

import sys

logger = logging.getLogger('chalmers')

def log_unhandled_exception(*exc_info):
    logger.error('', exc_info=exc_info)
    sys.exit(1)

def setup_logging(level, color=None, logfile='cli.log', short_tb=()):


    if not exists(dirs.user_log_dir): makedirs(dirs.user_log_dir)


    logger.setLevel(logging.DEBUG)

    error_logfile = join(dirs.user_log_dir, logfile)
    hndlr = RotatingFileHandler(error_logfile, maxBytes=10 * (1024 ** 2), backupCount=5,)
    hndlr.setLevel(logging.INFO)
    logger.addHandler(hndlr)

    shndlr = MyStreamHandler(color=color, short_tb=short_tb)
    shndlr.setLevel(level)
    logger.addHandler(shndlr)

    sys.excepthook = log_unhandled_exception




"""
Install windows services
"""
import getpass
import logging

from chalmers import errors
from chalmers.event_dispatcher import send_action
from chalmers.windows.install import get_service_name, is_installed, is_running
from chalmers.windows.install import instart
from win32com.shell import shell
from win32serviceutil import RemoveService, StopService
from chalmers.program_manager import ProgramManager
from subprocess import Popen

log = logging.getLogger(__name__)

try:
    input = raw_input
except NameError:
    pass
def bool_input(prompt, default=True):
        default_str = '[Y|n]' if default else '[y|N]'
        while 1:
            inpt = input('%s %s: ' % (prompt, default_str))
            if inpt.lower() in ['y', 'yes'] and not default:
                return True
            elif inpt.lower() in ['', 'n', 'no'] and not default:
                return False
            elif inpt.lower() in ['', 'y', 'yes']:
                return True
            elif inpt.lower() in ['n', 'no']:
                return False
            else:
                sys.stderr.write('please enter yes or no\n')

def run_as_admin(args, cmd):


    if args.username and args.username.lower() != getpass.getuser().lower():
        raise errors.ChalmersError("Can not use --username option when not an admin")

    admin_user= "Administrator"
    log.warning("Manging windows services requires admin privleges")
    if not bool_input("would you like to run this command as user '%s'" % admin_user):
        log.error("Exiting")
        return 

    cmd = ["runas", "/noprofile", "/env", "/user:%s" % admin_user, 
          cmd]

    p0 = Popen(cmd)
    if p0.wait():
        raise errors.ChalmersError("Command 'runas' did not complete successfully")


def main(args):

    if not shell.IsUserAnAdmin():
        run_as_admin(args,"chalmers install-service --username %s" % getpass.getuser())
        return  

    log.info("Your password is required by the windows service manager to launch"
             "The chalmers service at login")
    password = getpass.getpass(b"Password for %s: " % args.username)

    instart('.\\%s' % args.username, password)

def main_uninstall(args):

    if not shell.IsUserAnAdmin():
        run_as_admin(args, "chalmers uninstall-service --username %s" % getpass.getuser())
        return  

    service_name = get_service_name(args.username)

    if is_running(args.username):
        log.info("Service is running, stopping service %s" % service_name)
        StopService(service_name)

    if is_installed(args.username):
        RemoveService(service_name)
        log.info("Uninstalled windows service '%s'" % service_name)
    else:
        log.error("Windows service '%s' is not insatlled" % service_name)

def main_status(args):

    service_name = get_service_name(args.username)

    log.info("Status for service '%s'" % service_name)

    if is_installed(args.username):
        log.info("service '%s' is installed" % service_name)
    else:
        log.error("service '%s' is not installed" % service_name)
        return

    if is_running(args.username):
        log.info("service '%s' is running" % service_name)
    else:
        log.error("service '%s' is not running" % service_name)
        return

    pid = send_action(ProgramManager.NAME, "ping")
    log.info("Chalmers manger pid is %s" % pid)



from contextlib import contextmanager


class PosixColor(object):
    WHITE = 97
    YELLO = 93
    BLUE = 94
    GREEN = 92
    RED = 91
    BOLD = 1

    DEFAULT = 0

    BACKGROUND_COLORS = range(40, 48) + [100, 102, 104, 105, 106]

    def __init__(self, text, colors):
        self.text = text
        self.colors = colors

    @contextmanager
    def __call__(self, stream):
        for c in self.colors:
            stream.write('\033[%im' % c)
        try:
            yield self.text
        finally:
            stream.write('\033[%im' % self.DEFAULT)




from contextlib import contextmanager

import win32console

std_output_hdl = win32console.GetStdHandle(win32console.STD_OUTPUT_HANDLE)

class NTColor(object):
    YELLO = 14
    BLUE = 11
    GREEN = 10
    RED = 12
    BOLD = 0
    WHITE = 15

    DEFAULT = 15

    BACKGROUND_COLORS = [c<<4 for c in range(1,10)]

    def __init__(self, text, colors):
        self.text = text
        self.colors = colors

    @contextmanager
    def __call__(self, stream):
        c = reduce(lambda a,b: a|b, self.colors)
        std_output_hdl.SetConsoleTextAttribute(c)
        try:
            yield self.text
        finally:
            std_output_hdl.SetConsoleTextAttribute(self.DEFAULT)



from setuptools import setup, find_packages
import os

install_requires = ['psutil']

if os.name == 'nt':
    install_requires.append('pywin32')

setup(
    name='chalmers',
    version="0.1.1",
    author='Continuum Analytics',
    author_email='srossross@gmail.com',
    url='http://github.com/binstar/chalmers',
    packages=find_packages(),
    install_requires=install_requires,
    entry_points={
          'console_scripts': [
              'chalmers = chalmers.scripts.chalmers_main:main',
              ]
                 },
)




'''
Start a program.  
The program must be defined with the chalmers 'run' command first

example:

    chalmers start server1
'''
from __future__ import unicode_literals, print_function

from argparse import RawDescriptionHelpFormatter
import logging
import sys

from chalmers.program import Program
from chalmers.program_manager import ProgramManager


log = logging.getLogger('chalmers.start')


def main(args):

    if args.all:
        programs = list(Program.find_for_user())
    else:
        programs = [Program(name) for name in args.names]

    if args.daemon:
        for prog in programs:
            print("Starting program %s (daemon:%s) ... " % (prog.name, args.daemon), end=''); sys.stdout.flush()
            prog.start(args.daemon)
            print("started")
    else:
        mgr = ProgramManager(exit_on_first_failure=True, use_color=args.color)

        for prog in programs:
            mgr.dispatch_start(prog.name)


        for process in mgr.processes:
            process.join()


def restart_main(args):

    if args.all:
        programs = Program.find_for_user()
    else:
        programs = [Program(name) for name in args.names]

    for prog in programs:
        prog.restart()



def add_parser(subparsers):
    parser = subparsers.add_parser('start',
                                      help='Start a program',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter)

    parser.add_argument('names', nargs='*', metavar='PROG',
                        help='Names of the programs to start')
    parser.add_argument('-w', '--wait', '--no-daemon', action='store_false', dest='daemon',
                        help='Wait for program to exit')
    parser.add_argument('-d', '--daemon', action='store_true', dest='daemon', default=True,
                        help='Run program as daemon')
    parser.add_argument('-a', '--all', action='store_true',
                        help='start all programs')

    parser.set_defaults(main=main)

    parser = subparsers.add_parser('restart',
                                      help='Restart a program',
                                      description=__doc__)

    parser.add_argument('names', nargs='*', metavar='PROG',
                        help='Names of the programs to start')
    parser.add_argument('-a', '--all', action='store_true',
                        help='start all programs')

    parser.set_defaults(main=restart_main)



'''
Stop/Pause/Unpause a program

Stopping a program will send a signal to the program. The signal can be set by:

    chalmers set server1 stopsignal=SIGTERM
 
When paused, a program will not be started at system boot 
'''

from __future__ import unicode_literals, print_function

from argparse import RawDescriptionHelpFormatter
import logging
import sys

from chalmers import errors
from chalmers.program import Program


log = logging.getLogger('chalmers.stop')

def main(args):

    if args.all:
        programs = Program.find_for_user()
    else:
        programs = [Program(name) for name in args.names]

    for prog in programs:
        if prog.is_running:
            print("Stopping program %s ..." % prog.name, end=''); sys.stdout.flush()
            try:
                prog.stop()
            except errors.StateError as err:
                log.error(err.message)
            else:
                print("stopped ")
        else:
            print("Program %s is already stopped" % prog.name)

def pause_main(args):

    if args.all:
        programs = Program.find_for_user()
    else:
        programs = [Program(name) for name in args.names]

    for prog in programs:
        log.info("Pausing program %s" % (prog.name))
        if prog.is_running:
            log.warn("%s is running and will not restart on system reboot" % (prog.name))

        prog.update_state(paused=True)

def unpause_main(args):

    if args.all:
        programs = Program.find_for_user()
    else:
        programs = [Program(name) for name in args.names]

    for prog in programs:
        log.info("Unpausing program %s" % (prog.name))
        prog.update_state(paused=False)
        if not prog.is_running:
            log.warn("%s is not running and will start on system reboot" % (prog.name))


def add_parser(subparsers):
    parser = subparsers.add_parser('stop',
                                   help='Stop running a command',
                                   description=__doc__,
                                   formatter_class=RawDescriptionHelpFormatter)

    parser.add_argument('names', nargs='*', metavar='PROG')
    parser.add_argument('-a', '--all', action='store_true')
    parser.set_defaults(main=main)

    parser = subparsers.add_parser('pause',
                                      help='Pause program (don\'t run on system boot)',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter)

    parser.add_argument('names', nargs='*', metavar='PROG')
    parser.add_argument('-a', '--all', action='store_true')

    parser.set_defaults(main=pause_main)

    parser = subparsers.add_parser('unpause',
                                      help='Unpause program (run on system boot)',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter)

    parser.add_argument('names', nargs='*', metavar='PROG')
    parser.add_argument('-a', '--all', action='store_true')

    parser.set_defaults(main=unpause_main)



import logging
import os

from chalmers import errors
from chalmers.event_dispatcher import send_action

from .base import ProgramBase
from chalmers.utils.daemonize import daemonize


log = logging.getLogger(__name__)


def stop_process(signum, frame):
    """
    Signal handler to raise StopProcess exception
    """
    log.debug("Process recieved signal %s" % signum)
    raise errors.StopProcess()

class PosixProgram(ProgramBase):

    @property
    def is_running(self):
        "Check For the existence of a pid"
        pid = self.state.get('pid')
        if not pid:
            return False
        try:
            os.kill(pid, 0)
        except OSError:
            return False
        else:
            return True

    def start_as_service(self):
        """
        Run this program in a new background process

        posix only
        """

        daemonize(self.start_sync)


    def clear_socket(self):
        'Remove socket file'
        if os.path.exists(self.addr):
            log.debug("Removing socket file %s" % self.addr)
            os.unlink(self.addr)


    def stop(self):
        try:
            ProgramBase.stop(self)
        finally:
            self.clear_socket()




'''
Chalmers command line utility
'''
from __future__ import print_function, unicode_literals

from argparse import ArgumentParser
import logging

from chalmers import __version__ as version
from chalmers.commands import sub_commands
from chalmers.errors import ChalmersError, ShowHelp
from chalmers.utils.logutil import setup_logging


logger = logging.getLogger('chalmers')


def main(args=None, exit=True):


    parser = ArgumentParser(description=__doc__)
    parser.add_argument('--show-traceback', action='store_true')
    parser.add_argument('-v', '--verbose',
                        action='store_const', help='print debug information ot the console',
                        dest='log_level',
                        default=logging.INFO, const=logging.DEBUG)
    parser.add_argument('-q', '--quiet',
                        action='store_const', help='Only show warnings or errors the console',
                        dest='log_level', const=logging.WARNING)
    parser.add_argument('-V', '--version', action='version',
                        version="%%(prog)s Command line client (version %s)" % (version,))
    parser.add_argument('--color', action='store_true', default=None,
                        help='always display with colors')
    parser.add_argument('--no-color', action='store_false', dest='color',
                        help='never display with colors')
    subparsers = parser.add_subparsers(help='commands')

    for command in sub_commands():
        command.add_parser(subparsers)

    args = parser.parse_args(args)

    short_tb = ()

    if not args.show_traceback:
        short_tb = (ChalmersError, KeyboardInterrupt)

    setup_logging(args.log_level, args.color, short_tb=short_tb)

    try:
        return args.main(args)

    except ShowHelp as err:
        args.sub_parser.print_help()
        if exit:
            raise SystemExit(1)
        else:
            return 1

if __name__ == "__main__":
    main()
